#ifndef DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_BASEPRICEENTRY_INC
#define DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_BASEPRICEENTRY_INC

#include <string>
#include <stdint.h>

#include "dxapi/dxapi.h"
#include "dfp/DecimalNative.hpp"
#include "Dx_BaseEntry.h"

using namespace std;
using namespace DxApi;

using namespace deltix::dfp;

namespace dx_BITTREX::deltix::timebase::api::messages::universal {

class Dx_BasePriceEntry : public Dx_BaseEntry {
private:
    std::string __class_type__ = "deltix.timebase.api.messages.universal.BasePriceEntry";
    std::string __class_guid__ = "64406423fb2b4ajpjsoe_18";

public:
    virtual std::string & getTypeName() { return __class_type__; }
    virtual std::string & getGuid() { return __class_guid__; }

    virtual void decode(DataReader &reader) {
        Dx_BaseEntry::decode(reader);
        numberOfOrders_ = reader.readInt64();
        has_participantId_ = reader.readUTF8(participantId_);
        price_ = Decimal64::fromUnderlying((uint64_t) reader.readInt64());
        has_quoteId_ = reader.readUTF8(quoteId_);
        size_ = Decimal64::fromUnderlying((uint64_t) reader.readInt64());
    }

    virtual void encode(DataWriter &writer) {
        Dx_BaseEntry::encode(writer);
        writer.writeInt64(numberOfOrders_);
        writer.writeUTF8(has_participantId_ ? &participantId_ : NULL);
        writer.writeUInt64(price_.toUnderlying());
        writer.writeUTF8(has_quoteId_ ? &quoteId_ : NULL);
        writer.writeUInt64(size_.toUnderlying());
    }

    inline void clear() {
        Dx_BaseEntry::clear();
        numberOfOrders_ = DxApi::INT64_NULL;
        participantId_.clear();
        has_participantId_ = false;
        price_ = D64_NULL;
        quoteId_.clear();
        has_quoteId_ = false;
        size_ = D64_NULL;
    }

    inline void setNumberOfOrders(int64_t value) {
        this->numberOfOrders_ = value;
    }

    inline void setParticipantId(const string &value) {
        this->participantId_ = value;
        has_participantId_ = true;
    }

    inline void setPrice(Decimal64 value) {
        this->price_ = value;
    }

    inline void setQuoteId(const string &value) {
        this->quoteId_ = value;
        has_quoteId_ = true;
    }

    inline void setSize(Decimal64 value) {
        this->size_ = value;
    }

    virtual string toString() {
        stringstream ss;
        ss << Dx_BaseEntry::toString() << ", ";
        ss << "  numberOfOrders_: "; 
        if (numberOfOrders_ == DxApi::INT64_NULL) { ss << "[NULL]"; } else { ss << numberOfOrders_; }
        ss << ", ";
        ss << "  participantId_: "; 
        if (!has_participantId_) { ss << "[NULL]"; } else { ss << participantId_; }
        ss << ", ";
        ss << "  price_: "; 
        if (price_ == D64_NULL) { ss << "[NULL]"; } else { ss << price_; }
        ss << ", ";
        ss << "  quoteId_: "; 
        if (!has_quoteId_) { ss << "[NULL]"; } else { ss << quoteId_; }
        ss << ", ";
        ss << "  size_: "; 
        if (size_ == D64_NULL) { ss << "[NULL]"; } else { ss << size_; }
        return ss.str();
    };

    int64_t numberOfOrders_;
    string participantId_;
    Decimal64 price_;
    string quoteId_;
    Decimal64 size_;

    bool has_participantId_ = false;
    bool has_quoteId_ = false;
};

} // namespace dx_BITTREX::deltix::timebase::api::messages::universal

#endif /* DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_BASEPRICEENTRY_INC */
