#ifndef DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_TRADEENTRY_INC
#define DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_TRADEENTRY_INC

#include <string>
#include <stdint.h>

#include "dxapi/dxapi.h"
#include "dfp/DecimalNative.hpp"
#include "Dx_BaseEntry.h"

using namespace std;
using namespace DxApi;

using namespace deltix::dfp;

namespace dx_BITTREX::deltix::timebase::api::messages::universal {

class Dx_TradeEntry : public Dx_BaseEntry {
private:
    std::string __class_type__ = "deltix.timebase.api.messages.universal.TradeEntry";
    std::string __class_guid__ = "64406423fb2b4ajpjsoe_16";

public:
    virtual std::string & getTypeName() { return __class_type__; }
    virtual std::string & getGuid() { return __class_guid__; }

    virtual void decode(DataReader &reader) {
        Dx_BaseEntry::decode(reader);
        buyerNumberOfOrders_ = reader.readInt64();
        has_buyerOrderId_ = reader.readUTF8(buyerOrderId_);
        has_buyerParticipantId_ = reader.readUTF8(buyerParticipantId_);
        has_condition_ = reader.readUTF8(condition_);
        has_matchId_ = reader.readUTF8(matchId_);
        price_ = Decimal64::fromUnderlying((uint64_t) reader.readInt64());
        sellerNumberOfOrders_ = reader.readInt64();
        has_sellerOrderId_ = reader.readUTF8(sellerOrderId_);
        has_sellerParticipantId_ = reader.readUTF8(sellerParticipantId_);
        side_ = reader.readEnum8();
        size_ = Decimal64::fromUnderlying((uint64_t) reader.readInt64());
        tradeType_ = reader.readEnum8();
    }

    virtual void encode(DataWriter &writer) {
        Dx_BaseEntry::encode(writer);
        writer.writeInt64(buyerNumberOfOrders_);
        writer.writeUTF8(has_buyerOrderId_ ? &buyerOrderId_ : NULL);
        writer.writeUTF8(has_buyerParticipantId_ ? &buyerParticipantId_ : NULL);
        writer.writeUTF8(has_condition_ ? &condition_ : NULL);
        writer.writeUTF8(has_matchId_ ? &matchId_ : NULL);
        writer.writeUInt64(price_.toUnderlying());
        writer.writeInt64(sellerNumberOfOrders_);
        writer.writeUTF8(has_sellerOrderId_ ? &sellerOrderId_ : NULL);
        writer.writeUTF8(has_sellerParticipantId_ ? &sellerParticipantId_ : NULL);
        writer.writeEnum8(side_);
        writer.writeUInt64(size_.toUnderlying());
        writer.writeEnum8(tradeType_);
    }

    inline void clear() {
        Dx_BaseEntry::clear();
        buyerNumberOfOrders_ = DxApi::INT64_NULL;
        buyerOrderId_.clear();
        has_buyerOrderId_ = false;
        buyerParticipantId_.clear();
        has_buyerParticipantId_ = false;
        condition_.clear();
        has_condition_ = false;
        matchId_.clear();
        has_matchId_ = false;
        price_ = D64_NULL;
        sellerNumberOfOrders_ = DxApi::INT64_NULL;
        sellerOrderId_.clear();
        has_sellerOrderId_ = false;
        sellerParticipantId_.clear();
        has_sellerParticipantId_ = false;
        side_ = DxApi::ENUM_NULL;
        size_ = D64_NULL;
        tradeType_ = DxApi::ENUM_NULL;
    }

    inline void setBuyerNumberOfOrders(int64_t value) {
        this->buyerNumberOfOrders_ = value;
    }

    inline void setBuyerOrderId(const string &value) {
        this->buyerOrderId_ = value;
        has_buyerOrderId_ = true;
    }

    inline void setBuyerParticipantId(const string &value) {
        this->buyerParticipantId_ = value;
        has_buyerParticipantId_ = true;
    }

    inline void setCondition(const string &value) {
        this->condition_ = value;
        has_condition_ = true;
    }

    inline void setMatchId(const string &value) {
        this->matchId_ = value;
        has_matchId_ = true;
    }

    inline void setPrice(Decimal64 value) {
        this->price_ = value;
    }

    inline void setSellerNumberOfOrders(int64_t value) {
        this->sellerNumberOfOrders_ = value;
    }

    inline void setSellerOrderId(const string &value) {
        this->sellerOrderId_ = value;
        has_sellerOrderId_ = true;
    }

    inline void setSellerParticipantId(const string &value) {
        this->sellerParticipantId_ = value;
        has_sellerParticipantId_ = true;
    }

    inline void setSide(int value) {
        this->side_ = value;
    }

    inline void setSize(Decimal64 value) {
        this->size_ = value;
    }

    inline void setTradeType(int value) {
        this->tradeType_ = value;
    }

    virtual string toString() {
        stringstream ss;
        ss << Dx_BaseEntry::toString() << ", ";
        ss << "  buyerNumberOfOrders_: "; 
        if (buyerNumberOfOrders_ == DxApi::INT64_NULL) { ss << "[NULL]"; } else { ss << buyerNumberOfOrders_; }
        ss << ", ";
        ss << "  buyerOrderId_: "; 
        if (!has_buyerOrderId_) { ss << "[NULL]"; } else { ss << buyerOrderId_; }
        ss << ", ";
        ss << "  buyerParticipantId_: "; 
        if (!has_buyerParticipantId_) { ss << "[NULL]"; } else { ss << buyerParticipantId_; }
        ss << ", ";
        ss << "  condition_: "; 
        if (!has_condition_) { ss << "[NULL]"; } else { ss << condition_; }
        ss << ", ";
        ss << "  matchId_: "; 
        if (!has_matchId_) { ss << "[NULL]"; } else { ss << matchId_; }
        ss << ", ";
        ss << "  price_: "; 
        if (price_ == D64_NULL) { ss << "[NULL]"; } else { ss << price_; }
        ss << ", ";
        ss << "  sellerNumberOfOrders_: "; 
        if (sellerNumberOfOrders_ == DxApi::INT64_NULL) { ss << "[NULL]"; } else { ss << sellerNumberOfOrders_; }
        ss << ", ";
        ss << "  sellerOrderId_: "; 
        if (!has_sellerOrderId_) { ss << "[NULL]"; } else { ss << sellerOrderId_; }
        ss << ", ";
        ss << "  sellerParticipantId_: "; 
        if (!has_sellerParticipantId_) { ss << "[NULL]"; } else { ss << sellerParticipantId_; }
        ss << ", ";
        ss << "  side_: "; 
        if (side_ == DxApi::ENUM_NULL) { ss << "[NULL]"; } else { ss << side_; }
        ss << ", ";
        ss << "  size_: "; 
        if (size_ == D64_NULL) { ss << "[NULL]"; } else { ss << size_; }
        ss << ", ";
        ss << "  tradeType_: "; 
        if (tradeType_ == DxApi::ENUM_NULL) { ss << "[NULL]"; } else { ss << tradeType_; }
        return ss.str();
    };

    int64_t buyerNumberOfOrders_;
    string buyerOrderId_;
    string buyerParticipantId_;
    string condition_;
    string matchId_;
    Decimal64 price_;
    int64_t sellerNumberOfOrders_;
    string sellerOrderId_;
    string sellerParticipantId_;
    int side_;
    Decimal64 size_;
    int tradeType_;

    bool has_buyerOrderId_ = false;
    bool has_buyerParticipantId_ = false;
    bool has_condition_ = false;
    bool has_matchId_ = false;
    bool has_sellerOrderId_ = false;
    bool has_sellerParticipantId_ = false;
};

} // namespace dx_BITTREX::deltix::timebase::api::messages::universal

#endif /* DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_TRADEENTRY_INC */
