#include <iostream>
#include <vector>
#include <ctime>
#include <chrono>

#include "dxapi/dxapi.h"
#include "dfp/DecimalNative.hpp"

#include "codecs/BITTREX/Dx_BITTREXStreamCodec.h"
#include "codecs/BITTREX/Dx_SecurityStatusMessage.h"
#include "codecs/BITTREX/Dx_ConnectionStatusChangeMessage.h"
#include "codecs/BITTREX/Dx_BittrexPackageHeader.h"

using namespace std;
using namespace DxApi;

static int64_t now_ns();

void writestream() {
    try {
        // Create timebase connection
        unique_ptr<TickDb> db(TickDb::createFromUrl("dxtick://localhost:8001"));

        // Open in read-write mode
        db->open(false);

        // Get stream
        TickStream *stream = db->getStream("BITTREX");

        // Create stream loader
        LoadingOptions opt;
        shared_ptr<TickLoader> loader(stream->createLoader(opt));
        dx_BITTREX::Dx_BITTREXStreamEncoder encoder(loader);

        dx_BITTREX::deltix::timebase::api::messages::status::Dx_SecurityStatusMessage dx_securitystatusmessage;
        dx_BITTREX::deltix::timebase::api::messages::service::Dx_ConnectionStatusChangeMessage dx_connectionstatuschangemessage;
        dx_BITTREX::deltix::qsrv::hf::plugins::data::bittrex::types::Dx_BittrexPackageHeader dx_bittrexpackageheader;
        

        // Create instruments
        auto instrument = loader->getInstrumentId(InstrumentIdentity(DxApi::InstrumentType::EQUITY, "DLTX"));

        for (int sent = 0; sent < 10; ++sent) {
            // Write fields of deltix.qsrv.hf.plugins.data.bittrex.types.BittrexPackageHeader:
            dx_bittrexpackageheader.clear();
            dx_bittrexpackageheader.timestamp = now_ns();
            dx_bittrexpackageheader.typeId = dx_BITTREX::Dx_BittrexPackageHeaderType;
            dx_bittrexpackageheader.entityId = instrument;
            dx_bittrexpackageheader.setCurrencyCode(1);
            dx_bittrexpackageheader.setOriginalTimestamp(1373321607447L /* = 2013-07-08 22:13:27.447 GMT */);
            dx_bittrexpackageheader.setSequenceNumber(1);
            dx_bittrexpackageheader.setSourceId("NY4");
            dx_bittrexpackageheader.setPackageType(0);


            auto entries = dx_bittrexpackageheader.getEntries();
            entries.resize(20);
            for (int i = 0; i < 20; ++i) {
                auto entry = new dx_BITTREX::deltix::timebase::api::messages::universal::Dx_L2EntryNew();
                entry->setLevel(i);
                entry->setSide(i > 10 ? 0 : 1);
                entry->setPrice(deltix::dfp::Decimal64(1.111 + i));
                entry->setSize(deltix::dfp::Decimal64(2.222 + i));

                entries[i] = shared_ptr<NativeMessage>(entry);
            }
            dx_bittrexpackageheader.setEntries(entries);

            encoder.send(dx_bittrexpackageheader);
            cout << "[SENT]: " << dx_bittrexpackageheader.toString() << endl;
        }

        // Close loader after send
        loader->close();
    } catch (exception &e) {
        cout << e.what() << endl;
    } catch (...) {
        cout << "System exception" << endl;
    }
}

static int64_t now_ns() {
    return chrono::duration_cast<chrono::nanoseconds>(chrono::system_clock::now().time_since_epoch()).count();
}